<?php

namespace Drupal\notifications_widget\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a block with list of notification items.
 *
 * @Block(
 *   id = "notification_widget_block",
 *   admin_label = @Translation("Notification widget block"),
 *   category = @Translation("Notifications widget")
 * )
 */
class NotificationsWidgetBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Drupal\Core\Session\AccountInterface definition.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The Database Connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration,
    $plugin_id,
    $plugin_definition,
    AccountInterface $current_user,
    Connection $database) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->currentUser = $current_user;
    $this->database    = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_user'),
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockAccess(AccountInterface $account) {
    return AccessResult::allowedIf($account->isAuthenticated());
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $connection = $this->database;
    // Get logged user session.
    $currentUser = $this->currentUser;

    $uid = $currentUser->id();
    $notificationType = 'Admin';
    $totalCount = 0;
    $unreadCount = 0;
    $notificationList = [];

    $query = $connection->select('notifications', 'n');
    $query->fields('n', [
      'id',
      'message',
      'status',
    ]);
    $query->orderBy('n.created', 'DESC');
    $res = $query->execute();

    while ($notification = $res->fetchObject()) {

      if (!empty($notification->message)) {
        $notificationList[] = [
          'id'      => $notification->id,
          'message' => $notification->message,
          'status'  => $notification->status,
        ];

        if ($notification->status == 0) {
          $unreadCount++;
        }

        $totalCount++;
      }
    }

    return [
      '#theme' => 'notifications_widget',
      '#uid' => $uid,
      '#notification_type' => $notificationType,
      '#total' => $totalCount,
      '#unread' => $unreadCount,
      '#notification_list' => $notificationList,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return 0;
  }

}
